@@ -14,6 +14,8 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include <QEvent>
 #include <QImage>
 #include <QMap>
+#include <QProcessEnvironment>
+#include <QProgressDialog>
 #include <QShortcut>
 #include <QString>
 #include <QUndoView>
@@ -24,8 +26,9 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include <KColorSchemeManager>
 #include <KSelectAction>
 #include <KXmlGuiWindow>
-#include <knewstuff_version.h>
-#include <kxmlgui_version.h>
+#include <kconfigwidgets_version.h>
+#include <kiconthemes_version.h>
+
 #include <mlt++/Mlt.h>
 #include <utility>
 
@@ -34,7 +37,8 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "jobs/abstracttask.h"
 #include "kdenlive_debug.h"
 #include "kdenlivecore_export.h"
-#include "pythoninterfaces/otioconvertions.h"
+#include "otio/otioexport.h"
+#include "otio/otioimport.h"
 #include "statusbarmessagelabel.h"
 #include "utils/gentime.h"
 
@@ -48,6 +52,7 @@ class KdenliveDoc;
 class Monitor;
 class Render;
 class RenderWidget;
+class ScopeManager;
 class TimelineTabs;
 class TimelineWidget;
 class TimelineContainer;
@@ -85,7 +90,7 @@ public:
      * If Url is present, it will be opened, otherwise, if openlastproject is
      * set, latest project will be opened. If no file is open after trying this,
      * a default new file will be created. */
-    void init(const QString &mltPath);
+    void init();
     ~MainWindow() override;
 
     /** @brief Cache for luma files thumbnails. */
@@ -100,8 +105,8 @@ public:
     QAction *addAction(const QString &name, const QString &text, const QObject *receiver, const char *member, const QIcon &icon = QIcon(),
                        const QKeySequence &shortcut = QKeySequence(), KActionCategory *category = nullptr);
     /** @brief Same as above, but takes a string for category to populate it with kdenliveCategoryMap */
-    QAction *addAction(const QString &name, const QString &text, const QObject *receiver, const char *member, const QIcon &icon,
-                       const QKeySequence &shortcut, const QString &category);
+    QAction *addAction(const QString &name, const QString &text, const QObject *receiver, const char *member, const QIcon &icon, const QKeySequence &shortcut,
+                       const QString &category);
 
     void processRestoreState(const QByteArray &state);
 
@@ -116,7 +121,7 @@ public:
      */
     QDockWidget *addDock(const QString &title, const QString &objectName, QWidget *widget, Qt::DockWidgetArea area = Qt::TopDockWidgetArea);
 
-    QUndoGroup *m_commandStack;
+    QUndoGroup *m_commandStack{nullptr};
     QUndoView *m_undoView;
     /** @brief holds info about whether movit is available on this system */
     bool m_gpuAllowed;
@@ -125,7 +130,8 @@ public:
     QList<QAction *> getExtraActions(const QString &name);
     /** @brief Returns true if docked widget is tabbed with another widget from its object name */
     bool isTabbedWith(QDockWidget *widget, const QString &otherWidget);
-    
+    bool isDockTabbedWith(QDockWidget *widget, QDockWidget *otherWidget);
+
     /** @brief Returns true if mixer widget is tabbed */
     bool isMixedTabbed() const;
 
@@ -133,25 +139,33 @@ public:
     TimelineWidget *getCurrentTimeline() const;
     /** @brief Returns a pointer to the timeline with @uuid */
     TimelineWidget *getTimeline(const QUuid uuid) const;
-    void closeTimeline(const QUuid &uuid);
+    void getSequenceProperties(const QUuid &uuid, QMap<QString, QString> &props);
+    void closeTimelineTab(const QUuid uuid, bool onDeletion);
     /** @brief Returns a list of opened tabs uuids */
     const QStringList openedSequences() const;
 
     /** @brief Returns true if a timeline widget is available */
     bool hasTimeline() const;
-    
+
     /** @brief Returns true if the timeline widget is visible */
     bool timelineVisible() const;
-    
+
     /** @brief Raise (show) the clip or project monitor */
     void raiseMonitor(bool clipMonitor);
 
-    /** @brief Raise (show) the project bin*/
-    void raiseBin();
+    /** @brief Raise (show) the project bin
+     * @param unconditionnaly if false, we won't raise the bin if docked with the project monitor */
+    void raiseBin(bool unconditionnaly = true);
+    /** @brief Give focus to the active timeline widget */
+    void focusTimeline();
     /** @brief Add a bin widget*/
-    void addBin(Bin *bin, const QString &binName = QString());
+    void addBin(Bin *bin, const QString &binName = QString(), bool updateCount = true);
+    /** @brief Clean current document references from all bins*/
+    void cleanBins();
     /** @brief Get the main (first) bin*/
     Bin *getBin();
+    /** @brief Block/Unblock all bin selection signals*/
+    void blockBins(bool block);
     /** @brief Get the active (focused) bin or first one if none is active*/
     Bin *activeBin();
     /** @brief Ensure all bin widgets are tabbed together*/
@@ -172,21 +186,28 @@ public:
     void showKeyBinding(const QString &text = QString());
     /** @brief Disable multicam mode if it was active */
     void disableMulticam();
-
-#if KNEWSTUFF_VERSION < QT_VERSION_CHECK(5, 98, 0)
-    /** @brief Instantiates a "Get Hot New Stuff" dialog.
-     * @param configFile configuration file for KNewStuff
-     * @return number of installed items */
-    int getNewStuff(const QString &configFile);
-#endif
+    /** @brief Get the folder id for project bins */
+    const QStringList extraBinIds() const;
+    /** @brief Load the project bins of a project */
+    void loadExtraBins(const QStringList binInfo);
+    void folderRenamed(const QString &binId, const QString &folderName);
+    /** @brief Seek timeline if it is active */
+    void seekIfCurrent(const QUuid uuid, int pos);
 
     /** @brief Check if the maximum cached data size is not exceeded. */
     void checkMaxCacheSize();
-    TimelineWidget *openTimeline(const QUuid &uuid, const QString &tabName, std::shared_ptr<TimelineItemModel> timelineModel, MonitorProxy *proxy);
+    TimelineWidget *openTimeline(const QUuid &uuid, int ix, const QString &tabName, std::shared_ptr<TimelineItemModel> timelineModel,
+                                 bool openInMonitor = true);
     /** @brief Bring a timeline tab in front. Returns false if no tab exists for this timeline. */
     bool raiseTimeline(const QUuid &uuid);
     void connectTimeline();
-    void disconnectTimeline(TimelineWidget *timeline);
+    void disconnectTimeline(TimelineWidget *timeline, bool onClose = false);
+    static QProcessEnvironment getCleanEnvironement();
+    ObjectId effectStackOwner();
+    bool effectIsMasterOnly(const QString &assetId) const;
+    void reloadAssetPanel();
+    /** @brief If any task is running, ask user before closing */
+    bool hasRunningTask() const;
 
 protected:
     /** @brief Closes the window.
@@ -194,6 +215,7 @@ protected:
      *     the operation requested (starting waiting jobs or saving file) fails,
      *     true otherwise */
     bool queryClose() override;
+    bool m_windowClosing{false};
     void closeEvent(QCloseEvent *) override;
     bool eventFilter(QObject *object, QEvent *event) override;
 
@@ -214,8 +236,10 @@ private:
     /** @brief Rebuild the dock menu according to existing dock widgets. */
     void updateDockMenu();
 
-    OtioConvertions m_otioConvertions;
+    OtioExport *m_otioExport{nullptr};
+    OtioImport *m_otioImport{nullptr};
     KColorSchemeManager *m_colorschemes;
+    ScopeManager *m_scopesManager{nullptr};
 
     QDockWidget *m_projectBinDock;
     QDockWidget *m_effectListDock;
@@ -242,16 +266,15 @@ private:
     QAction *m_compositeAction;
 
     TimelineTabs *m_timelineTabs{nullptr};
-    QVector <Bin*>m_binWidgets;
-
-    /** This list holds all the scopes used in Kdenlive, allowing to manage some global settings */
-    QList<QDockWidget *> m_gfxScopesList;
+    QVector<Bin *> m_binWidgets;
 
     KActionCategory *m_effectActions;
     KActionCategory *m_transitionActions;
     QMenu *m_effectsMenu;
     QMenu *m_transitionsMenu;
     QMenu *m_timelineContextMenu;
+    QMenu *m_binsListMenu{nullptr};
+    QMenu *m_scopesListMenu{nullptr};
     QList<QAction *> m_timelineClipActions;
     KDualAction *m_useTimelineZone;
 
@@ -290,6 +313,7 @@ private:
     QAction *m_zoomOut;
     QAction *m_loopZone;
     QAction *m_playZone;
+    QAction *m_playZoneFromCursor;
     QAction *m_loopClip;
     QAction *m_proxyClip;
     QAction *m_buttonSubtitleEditTool;
@@ -309,25 +333,24 @@ private:
     bool readOptions();
     void saveOptions();
 
-    void loadGenerators();
     QStringList m_pluginFileNames;
     QByteArray m_timelineState;
     void buildDynamicActions();
     void loadClipActions();
+    void loadContainerActions();
 
     QTime m_timer;
     KXMLGUIClient *m_extraFactory;
     bool m_themeInitialized{false};
     bool m_isDarkTheme{false};
     EffectBasket *m_effectBasket;
-    /** @brief Update widget style. */
-    void doChangeStyle();
+    QProgressDialog *m_loadingDialog;
 
 public Q_SLOTS:
     void slotReloadEffects(const QStringList &paths);
     Q_SCRIPTABLE void setRenderingProgress(const QString &url, int progress, int frame);
     Q_SCRIPTABLE void setRenderingFinished(const QString &url, int status, const QString &error);
-    Q_SCRIPTABLE void addProjectClip(const QString &url, const QString & folder = QStringLiteral("-1"));
+    Q_SCRIPTABLE void addProjectClip(const QString &url, const QString &folder = QStringLiteral("-1"));
     Q_SCRIPTABLE void addTimelineClip(const QString &url);
     Q_SCRIPTABLE void addEffect(const QString &effectId);
     Q_SCRIPTABLE void scriptRender(const QString &url);
@@ -337,6 +360,7 @@ public Q_SLOTS:
 
     void slotSwitchVideoThumbs();
     void slotSwitchAudioThumbs();
+    void appHelpActivated();
 
     void slotPreferences();
     void slotShowPreferencePage(Kdenlive::ConfigPage page, int option = -1);
@@ -351,7 +375,7 @@ public Q_SLOTS:
     void slotSwitchTimelineZone(bool toggled);
     /** @brief Open the online services search dialog. */
     void slotDownloadResources();
-    /** @brief Initialze the subtitle model on project load. */
+    /** @brief Initialize the subtitle model on project load. */
     void slotInitSubtitle(const QMap<QString, QString> &subProperties, const QUuid &uuid);
     /** @brief Display the subtitle track and initialize subtitleModel if necessary. */
     void slotEditSubtitle(const QMap<QString, QString> &subProperties = {});
@@ -368,10 +392,10 @@ public Q_SLOTS:
     void updateProjectPath(const QString &path);
     /** @brief Update compositing action to display current project setting. */
     void slotUpdateCompositeAction(bool enable);
-    /** @brief Update duration of projet in timeline toolbar. */
+    /** @brief Update duration of project in timeline toolbar. */
     void slotUpdateProjectDuration(int pos);
     /** @brief The current timeline selection zone changed... */
-    void slotUpdateZoneDuration(int duration);
+    void slotUpdateZoneDuration();
     /** @brief Remove all unused clips from the project. */
     void slotCleanProject();
     void slotEditProjectSettings(int ix = 0);
@@ -384,6 +408,16 @@ public Q_SLOTS:
     void slotUpdateDocumentState(bool modified);
     /** @brief Open the clip job management dialog */
     void manageClipJobs(AbstractTask::JOBTYPE type = AbstractTask::NOJOBTYPE, QWidget *parentWidget = nullptr);
+    /** @brief Deletes item in timeline, project tree or effect stack depending on focus. */
+    void slotDeleteItem();
+    /** @brief Export a subtitle file */
+    void slotExportSubtitle();
+    /** @brief Display current mouse pos */
+    void slotUpdateMousePosition(int pos, int duration = -1);
+    /** @brief Focus current timeline clip in bin and display its range */
+    void slotClipInProjectTree(ObjectId ownerId = ObjectId(), bool seekToStart = false);
+    /** @brief Normalize audio channels before displaying them */
+    void slotNormalizeAudioChannel(bool normalize);
 
 private Q_SLOTS:
     /** @brief Shows the shortcut dialog. */
@@ -392,7 +426,6 @@ private Q_SLOTS:
     /** @brief Reflects setting changes to the GUI. */
     void updateConfiguration();
     void slotConnectMonitors();
-    void slotUpdateMousePosition(int pos, int duration = -1);
     void slotSwitchMarkersComments();
     void slotSwitchSnap();
     void slotShowTimelineTags();
@@ -412,8 +445,6 @@ private Q_SLOTS:
      *
      * Adopted from Dolphin (src/statusbar/dolphinstatusbar.cpp) */
     void slotShowZoomSliderToolTip(int zoomlevel = -1);
-    /** @brief Deletes item in timeline, project tree or effect stack depending on focus. */
-    void slotDeleteItem();
     void slotAddClipMarker();
     void slotDeleteClipMarker(bool allowGuideDeletion = false);
     void slotDeleteAllClipMarkers();
@@ -425,6 +456,7 @@ private Q_SLOTS:
      * The comment is set to the current position (therefore not dialog).
      * This can be useful to mark something during playback. */
     void slotAddMarkerGuideQuickly();
+    void slotAddMarkerWithCategory();
     void slotCutTimelineClip();
     void slotCutTimelineAllClips();
     void slotInsertClipOverwrite();
@@ -437,6 +469,7 @@ private Q_SLOTS:
     void slotRemovePreviewRender();
     void slotClearPreviewRender(bool resetZones = true);
     void slotSelectTimelineClip();
+    void slotSelectTimelineZone();
     void slotSelectTimelineTransition();
     void slotDeselectTimelineClip();
     void slotDeselectTimelineTransition();
@@ -478,8 +511,8 @@ private Q_SLOTS:
     void slotResizeItemEnd();
     void configureNotifications();
     void slotSeparateAudioChannel();
-    /** @brief Normalize audio channels before displaying them */
-    void slotNormalizeAudioChannel();
+    /** @brief Toggle automatic fit track height */
+    void slotAutoTrackHeight(bool enable);
     void slotInsertTrack();
     void slotDeleteTrack();
     /** @brief Show context menu to switch current track target audio stream. */
@@ -490,15 +523,11 @@ private Q_SLOTS:
     /** @brief Select all clips in timeline. */
     void slotSelectAllTracks();
     void slotUnselectAllTracks();
-#if KXMLGUI_VERSION < QT_VERSION_CHECK(5, 98, 0)
-    void slotGetNewKeyboardStuff(QComboBox *schemesList);
-#endif
     void slotAutoTransition();
     void slotRunWizard();
     void slotGroupClips();
     void slotUnGroupClips();
     void slotEditItemDuration();
-    void slotClipInProjectTree();
     // void slotClipToProjectTree();
     void slotSplitAV();
     void slotSwitchClip();
@@ -528,8 +557,6 @@ private Q_SLOTS:
     /** @brief Focus the timecode widget of current monitor. */
     void slotFocusTimecode();
 
-    /** @brief The monitor informs that it needs (or not) to have frames sent by the renderer. */
-    void slotMonitorRequestRenderFrame(bool request);
     /** @brief Update project because the use of proxy clips was enabled / disabled. */
     void slotUpdateProxySettings();
     /** @brief Disable proxies for this project. */
@@ -546,8 +573,6 @@ private Q_SLOTS:
     void triggerKey(QKeyEvent *ev);
     /** @brief Update monitor overlay actions on monitor switch */
     void slotUpdateMonitorOverlays(int id, int code);
-    /** @brief Update widget style */
-    void slotChangeStyle(QAction *a);
     /** @brief Create temporary top track to preview an effect */
     void createSplitOverlay(std::shared_ptr<Mlt::Filter> filter);
     void removeSplitOverlay();
@@ -560,8 +585,6 @@ private Q_SLOTS:
     /** @brief Open Cached Data management dialog. */
     void slotManageCache();
     void showMenuBar(bool show);
-    /** @brief Change forced icon theme setting (asks for app restart). */
-    void forceIconSet(bool force);
     /** @brief Toggle current project's compositing mode. */
     void slotUpdateCompositing(bool checked);
     /** @brief Set timeline toolbar icon size. */
@@ -574,6 +597,8 @@ private Q_SLOTS:
     void slotGrabItem();
     /** @brief Collapse or expand current item (depending on focused widget: effet, track)*/
     void slotCollapse();
+    /** @brief Cycle zoom audio waveforms*/
+    void slotZoomWaveForm();
     /** @brief Save currently selected timeline clip as bin subclip*/
     void slotExtractClip();
     /** @brief Save currently selected timeline clip as bin subclip*/
@@ -592,8 +617,8 @@ private Q_SLOTS:
     void slotLockSubtitle();
     /** @brief Import a subtitle file */
     void slotImportSubtitle();
-    /** @brief Export a subtitle file */
-    void slotExportSubtitle();
+    /** @brief Display the subtitle manager widget */
+    void slotManageSubtitle();
     /** @brief Start a speech recognition on timeline zone */
     void slotSpeechRecognition();
     /** @brief Copy debug information like lib versions, gpu mode state,... to clipboard */
@@ -601,6 +626,8 @@ private Q_SLOTS:
     void slotRemoveBinDock(const QString &name);
     /** @brief Focus the guides list search line */
     void slotSearchGuide();
+    /** @brief Focus the bin search line */
+    void slotSearchBin();
     /** @brief Copy current timeline selection to a new sequence clip / Timeline tab */
     void slotCreateSequenceFromSelection();
 
@@ -612,15 +639,17 @@ Q_SIGNALS:
     void setRenderProgress(int);
     void displayMessage(const QString &, MessageType, int);
     void displaySelectionMessage(const QString &);
-    void displayProgressMessage(const QString &, MessageType, int);
+    void displayProgressMessage(const QString &, MessageType, int, bool canBeStopped = false);
     /** @brief Project profile changed, update render widget accordingly. */
     void updateRenderWidgetProfile();
     /** @brief Clear asset view if itemId is displayed. */
     void clearAssetPanel(int itemId = -1);
-    void assetPanelWarning(const QString service, const QString id, const QString message);
+    void assetPanelWarning(const QString service, const QString message, const QString log = QString());
     void adjustAssetPanelRange(int itemId, int in, int out);
     /** @brief Enable or disable the undo stack. For example undo/redo should not be enabled when dragging a clip in timeline or we risk corruption. */
     void enableUndo(bool enable);
-    bool focusTimeline(bool focus, bool highlight);
+    bool showTimelineFocus(bool focus, bool highlight);
     void removeBinDock(const QString &name);
+    /** @brief Connect a newly created dock to signals updating/hiding its title bar. */
+    void connectDockAfterInit(QDockWidget *);
 };
