@@ -11,7 +11,9 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "bin/projectfolder.h"
 #include "bin/projectitemmodel.h"
 #include "core.h"
+#include "dialogs/customcamcorderdialog.h"
 #include "dialogs/profilesdialog.h"
+#include "dialogs/wizard.h"
 #include "doc/kdenlivedoc.h"
 #include "kdenlivesettings.h"
 #include "mainwindow.h"
@@ -24,11 +26,9 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "xml/xml.hpp"
 
 #include "kdenlive_debug.h"
-#include "utils/KMessageBox_KdenliveCompat.h"
 #include <KIO/FileCopyJob>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <kio/directorysizejob.h>
 
 #include <QDir>
 #include <QFileDialog>
@@ -53,11 +53,10 @@ public:
     }
 };
 
-ProjectSettings::ProjectSettings(KdenliveDoc *doc, QMap<QString, QString> metadata, QStringList lumas, int videotracks, int audiotracks, int audiochannels,
+ProjectSettings::ProjectSettings(KdenliveDoc *doc, QMap<QString, QString> metadata, int videotracks, int audiotracks, int audiochannels,
                                  const QString & /*projectPath*/, bool readOnlyTracks, bool savedProject, QWidget *parent)
     : QDialog(parent)
     , m_savedProject(savedProject)
-    , m_lumas(std::move(lumas))
     , m_newProject(doc == nullptr)
 {
     setupUi(this);
@@ -107,11 +106,14 @@ ProjectSettings::ProjectSettings(KdenliveDoc *doc, QMap<QString, QString> metada
     connect(external_proxy, &QCheckBox::toggled, external_proxy_profile, &QComboBox::setEnabled);
     connect(external_proxy_profile, &QComboBox::currentTextChanged, this, &ProjectSettings::slotExternalProxyProfileChanged);
     slotExternalProxyChanged(external_proxy->checkState());
+    connect(manage_external, &QToolButton::clicked, this, &ProjectSettings::configureExternalProxies);
 
     QString currentProf;
     if (doc) {
         currentProf = pCore->getCurrentProfile()->path();
         proxy_box->setChecked(doc->useProxy());
+        m_lumas = doc->extractCompositionLumas();
+        m_externalFiles = doc->extractExternalEffectFiles();
         generate_proxy->setChecked(doc->getDocumentProperty(QStringLiteral("generateproxy")).toInt() != 0);
         proxy_minsize->setValue(doc->getDocumentProperty(QStringLiteral("proxyminsize")).toInt());
         m_proxyparameters = doc->getDocumentProperty(QStringLiteral("proxyparams"));
@@ -175,7 +177,7 @@ ProjectSettings::ProjectSettings(KdenliveDoc *doc, QMap<QString, QString> metada
     proxy_imageminsize->setEnabled(generate_imageproxy->isChecked());
     QString currentProfileParams;
     if (!m_previewparams.isEmpty() || !m_previewextension.isEmpty()) {
-        currentProfileParams = QString("%1;%2").arg(m_previewparams, m_previewextension);
+        currentProfileParams = QStringLiteral("%1;%2").arg(m_previewparams, m_previewextension);
     }
     m_tlPreviewProfiles = new EncodingTimelinePreviewProfilesChooser(this, true, currentProfileParams);
     preview_profile_box->addWidget(m_tlPreviewProfiles);
@@ -263,7 +265,7 @@ ProjectSettings::ProjectSettings(KdenliveDoc *doc, QMap<QString, QString> metada
     connect(add_metadata, &QAbstractButton::clicked, this, &ProjectSettings::slotAddMetadataField);
     connect(delete_metadata, &QAbstractButton::clicked, this, &ProjectSettings::slotDeleteMetadataField);
     add_metadata->setIcon(QIcon::fromTheme(QStringLiteral("list-add")));
-    delete_metadata->setIcon(QIcon::fromTheme(QStringLiteral("list-remove")));
+    delete_metadata->setIcon(QIcon::fromTheme(QStringLiteral("edit-delete-remove")));
 
     // Guides categories
     QWidget *guidesPage = tabWidget->widget(2);
@@ -332,6 +334,17 @@ void ProjectSettings::setExternalProxyProfileData(const QString &profileData)
     le_suffix_clip->setText(val6);
 }
 
+void ProjectSettings::configureExternalProxies()
+{
+    // We want to edit the profiles
+    CustomCamcorderDialog cd;
+    if (cd.exec() == QDialog::Accepted) {
+        // reload profiles
+        m_initialExternalProxyProfile = external_proxy_profile->currentData().toString();
+        loadExternalProxyProfiles();
+    }
+}
+
 void ProjectSettings::slotExternalProxyProfileChanged(const QString &)
 {
     setExternalProxyProfileData(external_proxy_profile->currentData().toString());
@@ -347,7 +360,7 @@ void ProjectSettings::slotDeleteUnused()
     QStringList toDelete;
     QStringList idsToDelete;
     QList<std::shared_ptr<ProjectClip>> clipList = pCore->projectItemModel()->getRootFolder()->childClips();
-    for (const std::shared_ptr<ProjectClip> &clip : qAsConst(clipList)) {
+    for (const std::shared_ptr<ProjectClip> &clip : std::as_const(clipList)) {
         if (!clip->isIncludedInTimeline()) {
             idsToDelete << clip->clipId();
             ClipType::ProducerType type = clip->clipType();
@@ -360,7 +373,7 @@ void ProjectSettings::slotDeleteUnused()
         }
     }
     // make sure our urls are not used in another clip
-    for (const std::shared_ptr<ProjectClip> &clip : qAsConst(clipList)) {
+    for (const std::shared_ptr<ProjectClip> &clip : std::as_const(clipList)) {
         if (clip->isIncludedInTimeline()) {
             QUrl url(clip->getOriginalUrl());
             if (url.isValid() && toDelete.contains(url.path())) toDelete.removeAll(url.path());
@@ -424,12 +437,16 @@ void ProjectSettings::slotUpdateFiles(bool cacheOnly)
     subtitles->setExpanded(true);
     int count = 0;
     QStringList allFonts;
-    for (const QString &file : qAsConst(m_lumas)) {
+    for (const QString &file : std::as_const(m_lumas)) {
         count++;
         new QTreeWidgetItem(images, QStringList() << file);
     }
+    for (const QString &file : std::as_const(m_externalFiles)) {
+        count++;
+        new QTreeWidgetItem(others, QStringList() << file);
+    }
     QList<std::shared_ptr<ProjectClip>> clipList = pCore->projectItemModel()->getRootFolder()->childClips();
-    for (const std::shared_ptr<ProjectClip> &clip : qAsConst(clipList)) {
+    for (const std::shared_ptr<ProjectClip> &clip : std::as_const(clipList)) {
         switch (clip->clipType()) {
         case ClipType::Color:
         case ClipType::Timeline:
@@ -444,9 +461,12 @@ void ProjectSettings::slotUpdateFiles(bool cacheOnly)
             break;
         }
         case ClipType::Text: {
-            new QTreeWidgetItem(texts, QStringList() << clip->clipUrl());
-            const QStringList imagefiles = TitleWidget::extractImageList(clip->getProducerProperty(QStringLiteral("xmldata")));
-            const QStringList fonts = TitleWidget::extractFontList(clip->getProducerProperty(QStringLiteral("xmldata")));
+            if (!clip->clipUrl().isEmpty()) {
+                new QTreeWidgetItem(texts, QStringList() << clip->clipUrl());
+            }
+            const QString titleData = clip->getProducerProperty(QStringLiteral("xmldata"));
+            const QStringList imagefiles = TitleWidget::extractImageList(titleData, pCore->currentDoc()->documentRoot());
+            const QStringList fonts = TitleWidget::extractFontList(titleData);
             for (const QString &file : imagefiles) {
                 new QTreeWidgetItem(images, QStringList() << file);
             }
@@ -455,23 +475,29 @@ void ProjectSettings::slotUpdateFiles(bool cacheOnly)
         }
         case ClipType::Audio:
             new QTreeWidgetItem(sounds, QStringList() << clip->clipUrl());
+            count++;
             break;
         case ClipType::Image:
             new QTreeWidgetItem(images, QStringList() << clip->clipUrl());
+            count++;
             break;
         case ClipType::Playlist: {
             new QTreeWidgetItem(playlists, QStringList() << clip->clipUrl());
+            count++;
             const QStringList files = extractPlaylistUrls(clip->clipUrl());
             for (const QString &file : files) {
                 new QTreeWidgetItem(others, QStringList() << file);
+                count++;
             }
             break;
         }
         case ClipType::Unknown:
             new QTreeWidgetItem(others, QStringList() << clip->clipUrl());
+            count++;
             break;
         default:
             new QTreeWidgetItem(videos, QStringList() << clip->clipUrl());
+            count++;
             break;
         }
     }
@@ -508,7 +534,7 @@ void ProjectSettings::slotUpdateFiles(bool cacheOnly)
 
 const QString ProjectSettings::selectedPreview() const
 {
-    return QString("%1;%2").arg(m_tlPreviewProfiles->currentParams(), m_tlPreviewProfiles->currentExtension());
+    return QStringLiteral("%1;%2").arg(m_tlPreviewProfiles->currentParams(), m_tlPreviewProfiles->currentExtension());
 }
 
 void ProjectSettings::accept()
@@ -567,7 +593,7 @@ QString ProjectSettings::selectedProfile() const
     return m_pw->selectedProfile();
 }
 
-QPair<int, int> ProjectSettings::tracks() const
+std::pair<int, int> ProjectSettings::tracks() const
 {
     return {video_tracks->value(), audio_tracks->value()};
 }
@@ -712,6 +738,8 @@ QStringList ProjectSettings::extractPlaylistUrls(const QString &path)
         }
     }
 
+    // TODO merge with similar logic in DocumentChecker
+
     // luma files for transitions
     files = doc.elementsByTagName(QStringLiteral("transition"));
     for (int i = 0; i < files.count(); ++i) {
@@ -782,14 +810,11 @@ QStringList ProjectSettings::extractSlideshowUrls(const QString &url)
 
 void ProjectSettings::slotExportToText()
 {
-    QFileDialog fd(this);
-    fd.setMimeTypeFilters(QStringList() << "text/plain");
-    if (fd.exec() != QDialog::Accepted) {
+    const QString savePath = QFileDialog::getSaveFileName(this, i18n("Save As"), QString(), i18n("Text File (*.txt)"));
+    if (savePath.isEmpty()) {
         return;
     }
 
-    const QString savePath = fd.selectedFiles().constFirst();
-
     QString text;
     text.append(i18n("Project folder: %1", project_folder->url().toLocalFile()) + '\n');
     text.append(i18n("Project profile: %1", m_pw->selectedProfile()) + '\n');
@@ -880,11 +905,12 @@ void ProjectSettings::loadProxyProfiles()
     QMapIterator<QString, QString> k(values);
     int ix = -1;
     proxy_profile->clear();
-    if (KdenliveSettings::vaapiEnabled() || KdenliveSettings::nvencEnabled()) {
-        proxy_profile->addItem(QIcon::fromTheme(QStringLiteral("speedometer")), i18n("Automatic"));
+    if (!KdenliveSettings::supportedHWCodecs().isEmpty()) {
+        proxy_profile->addItem(QIcon::fromTheme(QStringLiteral("speedometer")), i18n("Automatic (%1)", Wizard::getHWCodecFriendlyName()));
     } else {
         proxy_profile->addItem(i18n("Automatic"));
     }
+    const QStringList allHWCodecs = Wizard::codecs();
     while (k.hasNext()) {
         k.next();
         if (!k.key().isEmpty()) {
@@ -894,17 +920,21 @@ void ProjectSettings::loadProxyProfiles()
                 // this is the current profile
                 ix = proxy_profile->count();
             }
-            if (params.contains(QLatin1String("vaapi"))) {
-                proxy_profile->addItem(KdenliveSettings::vaapiEnabled() ? QIcon::fromTheme(QStringLiteral("speedometer"))
-                                                                        : QIcon::fromTheme(QStringLiteral("dialog-cancel")),
-                                       k.key(), k.value());
-            } else if (params.contains(QLatin1String("nvenc"))) {
-                proxy_profile->addItem(KdenliveSettings::nvencEnabled() ? QIcon::fromTheme(QStringLiteral("speedometer"))
-                                                                        : QIcon::fromTheme(QStringLiteral("dialog-cancel")),
-                                       k.key(), k.value());
-            } else {
-                proxy_profile->addItem(k.key(), k.value());
+            QString itemCodec;
+            QStringList values = params.split(QLatin1Char('-'));
+            for (auto &v : values) {
+                if (v.startsWith(QLatin1String("vcodec ")) || v.startsWith(QLatin1String("codec:v ")) || v.startsWith(QLatin1String("c:v "))) {
+                    itemCodec = v.section(QLatin1Char(' '), 1);
+                    break;
+                }
             }
+            if (!itemCodec.isEmpty() && allHWCodecs.contains(itemCodec)) {
+                if (KdenliveSettings::supportedHWCodecs().contains(itemCodec)) {
+                    proxy_profile->addItem(QIcon::fromTheme(QStringLiteral("speedometer")), k.key(), k.value());
+                }
+                continue;
+            }
+            proxy_profile->addItem(k.key(), k.value());
         }
     }
     if (ix == -1) {
@@ -923,8 +953,7 @@ void ProjectSettings::loadProxyProfiles()
 void ProjectSettings::loadExternalProxyProfiles()
 {
     // load proxy profiles
-    KConfig conf(QStringLiteral("externalproxies.rc"), KConfig::CascadeConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup group(&conf, "proxy");
+    KConfigGroup group(KSharedConfig::openConfig(QStringLiteral("externalproxies.rc"), KConfig::CascadeConfig, QStandardPaths::AppDataLocation), "proxy");
     QMap<QString, QString> values = group.entryMap();
     QMapIterator<QString, QString> k(values);
     int ix = -1;
