@@ -1,35 +1,18 @@
-/***************************************************************************
- *   Copyright (C) 2017 by Jean-Baptiste Mardelle                          *
- *   This file is part of Kdenlive. See www.kdenlive.org.                  *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) version 3 or any later version accepted by the       *
- *   membership of KDE e.V. (or its successor approved  by the membership  *
- *   of KDE e.V.), which shall act as a proxy defined in Section 14 of     *
- *   version 3 of the license.                                             *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2017 Jean-Baptiste Mardelle
+    SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+*/
 
-#ifndef TIMELINECONTROLLER_H
-#define TIMELINECONTROLLER_H
+#pragma once
 
 #include "definitions.h"
 #include "lib/audio/audioCorrelation.h"
 #include "timeline2/model/timelineitemmodel.hpp"
 
 #include <KActionCollection>
+#include <QApplication>
 #include <QDir>
 
-class PreviewManager;
 class QAction;
 class QQuickItem;
 
@@ -65,6 +48,10 @@ class TimelineController : public QObject
     Q_PROPERTY(bool useRuler READ useRuler NOTIFY useRulerChanged)
     Q_PROPERTY(bool scrollVertically READ scrollVertically NOTIFY scrollVerticallyChanged)
     Q_PROPERTY(int activeTrack READ activeTrack WRITE setActiveTrack NOTIFY activeTrackChanged)
+    Q_PROPERTY(QString audioZoomText READ audioZoomText NOTIFY audioThumbNormalizeChanged)
+    Q_PROPERTY(int activeSubLayer READ activeSubLayer WRITE setActiveSubLayer NOTIFY activeSubLayerChanged)
+    Q_PROPERTY(QVariantList subtitlesList READ subtitlesList NOTIFY subtitlesListChanged)
+    Q_PROPERTY(int maxSubLayer READ getMaxSubLayer WRITE setMaxSubLayer NOTIFY maxSubLayerChanged)
     Q_PROPERTY(QVariantList audioTarget READ audioTarget NOTIFY audioTargetChanged)
     Q_PROPERTY(int videoTarget READ videoTarget WRITE setVideoTarget NOTIFY videoTargetChanged)
 
@@ -87,23 +74,43 @@ class TimelineController : public QObject
     Q_PROPERTY(QColor lockedColor READ lockedColor NOTIFY colorsChanged)
     Q_PROPERTY(QColor selectionColor READ selectionColor NOTIFY colorsChanged)
     Q_PROPERTY(QColor groupColor READ groupColor NOTIFY colorsChanged)
+    Q_PROPERTY(bool subtitlesWarning READ subtitlesWarning NOTIFY subtitlesWarningChanged)
     Q_PROPERTY(bool subtitlesDisabled READ subtitlesDisabled NOTIFY subtitlesDisabledChanged)
     Q_PROPERTY(bool subtitlesLocked READ subtitlesLocked NOTIFY subtitlesLockedChanged)
+    Q_PROPERTY(int activeSubPosition MEMBER m_activeSubPosition NOTIFY activeSubtitlePositionChanged)
+    Q_PROPERTY(bool guidesLocked READ guidesLocked NOTIFY guidesLockedChanged)
+    Q_PROPERTY(bool autotrackHeight MEMBER m_autotrackHeight NOTIFY autotrackHeightChanged)
     Q_PROPERTY(QPoint effectZone MEMBER m_effectZone NOTIFY effectZoneChanged)
+    Q_PROPERTY(int trimmingMainClip READ trimmingMainClip NOTIFY trimmingMainClipChanged)
+    Q_PROPERTY(int multicamIn MEMBER multicamIn NOTIFY multicamInChanged)
 
 public:
     TimelineController(QObject *parent);
     ~TimelineController() override;
+
+    struct TargetTracksData
+    {
+        bool keep{false};
+        bool needsToBeApplied{false};
+        bool hasVideo{false};
+        QMap<int, QString> audioTargets;
+    };
     /** @brief Sets the model that this widgets displays */
     void setModel(std::shared_ptr<TimelineItemModel> model);
     std::shared_ptr<TimelineItemModel> getModel() const;
     void setRoot(QQuickItem *root);
     /** @brief Edit an item's in/out points with a dialog
      */
-    Q_INVOKABLE void editItemDuration(int itemId = -1);
+    Q_INVOKABLE void editItemDuration();
     /** @brief Edit a title clip with a title widget
      */
     Q_INVOKABLE void editTitleClip(int itemId = -1);
+    /** @brief Open a sequence timeline
+     */
+    Q_INVOKABLE void focusTimelineSequence(int id);
+    /** @brief Edit an animation with Glaxnimate
+     */
+    Q_INVOKABLE void editAnimationClip(int itemId = -1);
 
     /** @brief Returns the topmost track containing a selected item (-1 if selection is embty) */
     Q_INVOKABLE int selectedTrack() const;
@@ -112,8 +119,10 @@ public:
         @param type is the type of the object (clip or composition)
         @param select: true if the object should be selected and false if it should be deselected
         @param addToCurrent: if true, the object will be added to the new selection
+        @param showErrorMsg inform the user that no item was selected
+        @return false if no item was found under timeline cursor in active track
     */
-    void selectCurrentItem(ObjectType type, bool select, bool addToCurrent = false);
+    bool selectCurrentItem(KdenliveObjectType type, bool select, bool addToCurrent = false, bool showErrorMsg = true);
 
     /** @brief Select all timeline items
      */
@@ -136,10 +145,17 @@ public:
 
     /** @brief Show/hide audio record controls on a track
      */
-    Q_INVOKABLE void switchRecording(int trackId);
+    Q_INVOKABLE void switchRecording(int trackId = -1, bool record = true);
+    /** @brief Get the shortcut for an action
+     */
+    Q_INVOKABLE const QString getActionShortcut(const QString actionName);
     /** @brief Add recorded file to timeline
      */
     void finishRecording(const QString &recordedFile);
+    /** @brief Add given file to bin, and then insert it at current point in timeline
+        @param highlightClip If true, highlights the newly created clip in the bin as well
+     */
+    void addAndInsertFile(const QString &recordedFile, int tid, std::pair<int, int> recPosition, const bool isAudioClip, const bool highlightClip);
     /** @brief Open Kdenlive's config diablog on a defined page and tab
      */
     Q_INVOKABLE void showConfig(int page, int tab);
@@ -148,6 +164,10 @@ public:
      */
     Q_INVOKABLE bool hasActiveTracks() const;
 
+    /** @brief Returns the audio zoom factor, empty if none
+     */
+    Q_INVOKABLE QString audioZoomText() const;
+
     /** @brief returns current timeline's zoom factor
      */
     Q_INVOKABLE double scaleFactor() const;
@@ -165,6 +185,7 @@ public:
      */
     Q_INVOKABLE QVariantList audioTarget() const;
     Q_INVOKABLE QVariantList lastAudioTarget() const;
+    Q_INVOKABLE void keepAudioTargets(bool keep);
     Q_INVOKABLE const QString audioTargetName(int tid) const;
     Q_INVOKABLE int videoTarget() const;
     Q_INVOKABLE int hasAudioTarget() const;
@@ -172,6 +193,7 @@ public:
     Q_INVOKABLE bool hasVideoTarget() const;
     bool autoScroll() const;
     Q_INVOKABLE int activeTrack() const { return m_activeTrack; }
+    Q_INVOKABLE int trimmingMainClip() const { return m_trimmingMainClip; }
     Q_INVOKABLE QColor videoColor() const;
     Q_INVOKABLE QColor audioColor() const;
     Q_INVOKABLE QColor titleColor() const;
@@ -184,21 +206,29 @@ public:
     Q_INVOKABLE QColor lockedColor() const;
     Q_INVOKABLE QColor selectionColor() const;
     Q_INVOKABLE QColor groupColor() const;
+    Q_INVOKABLE int doubleClickInterval() const { return QApplication::doubleClickInterval(); }
     Q_INVOKABLE void showToolTip(const QString &info = QString()) const;
     Q_INVOKABLE void showKeyBinding(const QString &info = QString()) const;
     Q_INVOKABLE void showTimelineToolInfo(bool show) const;
+    /** @brief The model list for this timeline's subtitles */
+    Q_INVOKABLE QVariantList subtitlesList() const;
+    int getMaxSubLayer() const;
+    void setMaxSubLayer(int value);
+    /** @brief Returns true if the avfilter.subtiles filter is not found */
+    bool subtitlesWarning() const;
+    Q_INVOKABLE void subtitlesWarningDetails();
     void switchSubtitleDisable();
     bool subtitlesDisabled() const;
     void switchSubtitleLock();
     bool subtitlesLocked() const;
-    /** @brief Request a seek operation
-       @param position is the desired new timeline position
-     */
+    bool guidesLocked() const;
     int zoneIn() const { return m_zone.x(); }
     int zoneOut() const { return m_zone.y(); }
     void setZoneIn(int inPoint);
     void setZoneOut(int outPoint);
     void setZone(const QPoint &zone, bool withUndo = true);
+    /** @brief Adjust timeline zone to current selection */
+    void setZoneToSelection();
     /** @brief Request a seek operation
        @param position is the desired new timeline position
      */
@@ -225,7 +255,8 @@ public:
      * @return the ids of the inserted clips
      */
     Q_INVOKABLE QList<int> insertClips(int tid, int position, const QStringList &binIds, bool logUndo, bool refreshView);
-    Q_INVOKABLE void copyItem();
+    Q_INVOKABLE int copyItem();
+    std::pair<int, QString> getCopyItemData();
     Q_INVOKABLE bool pasteItem(int position = -1, int tid = -1);
     /** @brief Request inserting a new composition in timeline (dragged from compositions list)
        @param tid is the destination track
@@ -235,6 +266,15 @@ public:
        @return the id of the inserted composition
     */
     Q_INVOKABLE int insertComposition(int tid, int position, const QString &transitionId, bool logUndo);
+    /** @brief Request inserting a new mix in timeline (dragged from compositions list)
+       @param tid is the destination track
+       @param position is the timeline position (clip start of the second clip)
+       @param transitionId is the data describing the dropped composition
+    */
+    Q_INVOKABLE void insertNewMix(int tid, int position, const QString &transitionId);
+    /** @brief Returns the cut position if the composition is over a cut between 2 clips, -1 otherwise
+    */
+    Q_INVOKABLE int isOnCut(int cid) const;
     /** @brief Request inserting a new composition in timeline (dragged from compositions list)
        this function will check if there is a clip at insert point and
        adjust the composition length accordingly
@@ -244,7 +284,7 @@ public:
        @param logUndo if set to false, no undo object is stored
        @return the id of the inserted composition
     */
-    Q_INVOKABLE int insertNewComposition(int tid, int position, const QString &transitionId, bool logUndo);
+    Q_INVOKABLE int insertNewCompositionAtPos(int tid, int position, const QString &transitionId);
     Q_INVOKABLE int insertNewComposition(int tid, int clipId, int offset, const QString &transitionId, bool logUndo);
 
     /** @brief Request deletion of the currently selected clips
@@ -260,7 +300,10 @@ public:
      * only return the id if timeline cursor is inside item
      */
     int getMainSelectedItem(bool restrictToCurrentPos = true, bool allowComposition = false);
-    int getMainSelectedClip() const;
+    int getMainSelectedClip();
+    /** @brief Return the {position, track id} of current selection. Operates only on video items (or audio if audioPart is true)
+     */
+    std::pair<int, int> selectionPosition(int *aTracks, int *vTracks);
 
     /** @brief Do we want to display video thumbnails
      */
@@ -272,15 +315,15 @@ public:
     /** @brief Do we want to display audio thumbnails
      */
     Q_INVOKABLE bool showWaveforms() const;
-    /** @brief Insert a timeline track
+    /** @brief Invoke the GUI to add new timeline tracks
      */
-    Q_INVOKABLE void addTrack(int tid);
+    Q_INVOKABLE void beginAddTrack(int tid);
     /** @brief Remove multiple(or single) timeline tracks
      */
     Q_INVOKABLE void deleteMultipleTracks(int tid);
     /** @brief Show / hide audio rec controls in active track
      */
-    void switchTrackRecord(int tid = -1);
+    void switchTrackRecord(int tid = -1, bool monitor = false);
     /** @brief Group selected items in timeline
      */
     Q_INVOKABLE void groupSelection();
@@ -305,8 +348,8 @@ public:
     /** @brief Ask for edit timeline guide dialog
      */
     Q_INVOKABLE void editGuide(int frame = -1);
-    Q_INVOKABLE void moveGuide(int frame, int newFrame);
-    Q_INVOKABLE void moveGuideWithoutUndo(int frame, int newFrame);
+    Q_INVOKABLE void moveGuideById(int id, int newFrame);
+    Q_INVOKABLE int moveGuideWithoutUndo(int mid, int newFrame);
     /** @brief Move all guides in the given range
      * @param start the start point of the range in frames
      * @param end the end point of the range in frames
@@ -324,7 +367,7 @@ public:
 
     /** @brief Add a timeline guide
      */
-    Q_INVOKABLE void switchGuide(int frame = -1, bool deleteOnly = false);
+    Q_INVOKABLE void switchGuide(int frame = -1, bool deleteOnly = false, bool showGui = false);
     /** @brief Request monitor refresh
      */
     Q_INVOKABLE void requestRefresh();
@@ -337,10 +380,11 @@ public:
     Q_INVOKABLE void showTrackAsset(int trackId);
     /** @brief Adjust height of all similar (audio or video) tracks
     */
+    Q_INVOKABLE void adjustTrackHeight(int trackId, int height);
     Q_INVOKABLE void adjustAllTrackHeight(int trackId, int height);
     Q_INVOKABLE void collapseAllTrackHeight(int trackId, bool collapse, int collapsedHeight);
 
-    /** @brief Reset track @trackId height to default track height. Adjusts all tracks if @trackId == -1
+    /** @brief Reset track \@trackId height to default track height. Adjusts all tracks if \@trackId == -1
     */
     Q_INVOKABLE void defaultTrackHeight(int trackId);
 
@@ -348,6 +392,9 @@ public:
 
     Q_INVOKABLE int headerWidth() const;
     Q_INVOKABLE void setHeaderWidth(int width);
+    /** @brief Hide / show a timeline track
+     */
+    Q_INVOKABLE void hideTrack(int trackId, bool hide, bool allTracks = false);
 
     /** @brief Seek to next snap point
      */
@@ -364,10 +411,10 @@ public:
 
     /** @brief Set current item's start point to cursor position
      */
-    void setInPoint();
+    void setInPoint(bool ripple = false);
     /** @brief Set current item's end point to cursor position
      */
-    void setOutPoint();
+    void setOutPoint(bool ripple = false);
     /** @brief Return the project's tractor
      */
     Mlt::Tractor *tractor();
@@ -387,16 +434,29 @@ public:
 
     /** @brief Cuts the clip on current track at timeline position
      */
-    Q_INVOKABLE void cutClipUnderCursor(int position = -1, int track = -1);
+    Q_INVOKABLE void cutClipUnderCursor(int position = -1, int track = -1, int subLayer = -1);
     /** @brief Cuts all clips at timeline position
      */
     Q_INVOKABLE void cutAllClipsUnderCursor(int position = -1);
     /** @brief Request a spacer operation
      */
     Q_INVOKABLE int requestSpacerStartOperation(int trackId, int position);
+    /** @brief Returns the minimum available position for a spacer operation
+     */
+    Q_INVOKABLE int spacerMinPos() const;
+    Q_INVOKABLE int spacerMaxPos() const;
+    /** @brief Get a list of guides Id after a given frame
+     */
+    Q_INVOKABLE QVector<int> spacerSelection(int startFrame);
+    /** @brief Move a list of guides from a given offset
+     */
+    Q_INVOKABLE void spacerMoveGuides(const QVector<int> &ids, int offset);
+    /** @brief Get the position of the first marker in the list
+     */
+    Q_INVOKABLE int getGuidePosition(int ids);
     /** @brief Request a spacer operation
      */
-    Q_INVOKABLE bool requestSpacerEndOperation(int clipId, int startPosition, int endPosition, int affectedTrack, int guideStart = -1);
+    Q_INVOKABLE bool requestSpacerEndOperation(int clipId, int startPosition, int endPosition, int affectedTrack, const QVector<int> &selectedGuides = QVector<int>(), int guideStart = -1);
     /** @brief Request a Fade in effect for clip
      */
     Q_INVOKABLE void adjustFade(int cid, const QString &effectId, int duration, int initialDuration);
@@ -406,23 +466,28 @@ public:
      */
     Q_INVOKABLE void insertSpace(int trackId = -1, int frame = -1);
     Q_INVOKABLE void removeSpace(int trackId = -1, int frame = -1, bool affectAllTracks = false);
+    /** @brief Remove all spaces in a @trackId track after @frame position
+     */
+    void removeTrackSpaces(int trackId, int frame);
+    /** @brief Remove all clips in a @trackId track after @frame position
+     */
+    void removeTrackClips(int trackId, int frame);
     /** @brief If clip is enabled, disable, otherwise enable
      */
     Q_INVOKABLE void switchEnableState(std::unordered_set<int> selection = {});
     Q_INVOKABLE void addCompositionToClip(const QString &assetId, int clipId = -1, int offset = -1);
     Q_INVOKABLE void addEffectToClip(const QString &assetId, int clipId = -1);
+    Q_INVOKABLE void setEffectsEnabled(int clipId, bool enabled);
 
     Q_INVOKABLE void requestClipCut(int clipId, int position);
 
     /** @brief Extract (delete + remove space) current clip
      */
-    void extract(int clipId = -1);
+    void extract(int clipId = -1, bool singleSelectionMode = false);
     /** @brief Save current clip cut as bin subclip
      */
     void saveZone(int clipId = -1);
 
-    Q_INVOKABLE void splitAudio(int clipId);
-    Q_INVOKABLE void splitVideo(int clipId);
     Q_INVOKABLE void setAudioRef(int clipId = -1);
     Q_INVOKABLE void alignAudio(int clipId = -1);
     Q_INVOKABLE void urlDropped(QStringList droppedFile, int frame, int tid);
@@ -443,6 +508,8 @@ public:
     Q_INVOKABLE void removeEffectKeyframe(int cid, int frame);
     Q_INVOKABLE void updateEffectKeyframe(int cid, int oldFrame, int newFrame, const QVariant &normalizedValue = QVariant());
     Q_INVOKABLE bool hasKeyframeAt(int cid, int frame);
+    /** @brief Cycle vertical zooming of audio waveforms */
+    Q_INVOKABLE void zoomWaveform();
 
     /** @brief Make current timeline track active/inactive*/
     Q_INVOKABLE void switchTrackActive(int trackId = -1);
@@ -450,11 +517,12 @@ public:
     void switchAllTrackActive();
     /** @brief Make all tracks active or inactive */
     void makeAllTrackActive();
+    void switchTrackDisabled();
     void switchTrackLock(bool applyToAll = false);
     void switchTargetTrack();
 
     const QString getTrackNameFromIndex(int trackIndex);
-    /** @brief Seeks to selected clip start / end
+    /** @brief Seeks to selected clip start / end or, if none is selected, to the start / end of the clip under the cursor
      */
     void seekCurrentClip(bool seekToEnd = false);
     /** @brief Seeks to a clip start (or end) based on it's clip id
@@ -465,7 +533,7 @@ public:
     bool positionIsInItem(int id);
     /** @brief Returns the number of tracks (audioTrakcs, videoTracks)
      */
-    QPair<int, int>getTracksCount() const;
+    Q_INVOKABLE QPair<int, int> getAvTracksCount() const;
     /** @brief Request monitor refresh if item (clip or composition) is under timeline cursor
      */
     void refreshItem(int id);
@@ -483,15 +551,12 @@ public:
      */
     bool isActiveTrackAudio() const;
 
-    /** @brief Returns a list of all luma files used in the project
+    /** @brief Get the x,y position of the mouse in the timeline widget
      */
-    QStringList extractCompositionLumas() const;
-    /** @brief Returns a list of all external files used by effects in the timeline
-     */
-    QStringList extractExternalEffectFiles() const;
+    Q_INVOKABLE const QPoint getMousePosInTimeline() const;
     /** @brief Get the frame where mouse is positioned
      */
-    int getMousePos();
+    Q_INVOKABLE int getMousePos();
     /** @brief Get the frame where mouse is positioned
      */
     int getMouseTrack();
@@ -506,6 +571,8 @@ public:
      */
     bool compositionAutoTrack(int cid) const;
     const QString getClipBinId(int clipId) const;
+    /** @brief Move timeline cursor position to an item's start position.
+     */
     void focusItem(int itemId);
     /** @brief Create and display a split clip view to compare effect
      */
@@ -513,6 +580,21 @@ public:
     /** @brief Delete the split clip view to compare effect
      */
     void removeSplitOverlay();
+
+    /** @brief Limit the given offset to the max/min possible offset of the main trimming clip
+     *  @returns The bounded offset
+     */
+    int trimmingBoundOffset(int offset);
+    /** @brief @todo TODO */
+    bool slipProcessSelection(int mainClipId, bool addToSelection);
+    Q_INVOKABLE bool requestStartTrimmingMode(int clipId = -1, bool addToSelection = false, bool right = false);
+    Q_INVOKABLE void requestEndTrimmingMode();
+    Q_INVOKABLE void slipPosChanged(int offset);
+    Q_INVOKABLE void ripplePosChanged(int pos, bool right);
+
+    /** @brief @todo TODO */
+    Q_INVOKABLE int requestItemRippleResize(int itemId, int size, bool right, bool logUndo = true, int snapDistance = -1, bool allowSingleResize = false);
+
     /** @brief Add current timeline zone to preview rendering
      */
     void addPreviewRange(bool add);
@@ -535,16 +617,15 @@ public:
 
     /** @brief Load timeline preview from saved doc
      */
-    void loadPreview(const QString &chunks, const QString &dirty, const QDateTime &documentDate, int enable);
-    /** @brief Return document properties with added settings from timeline
-     */
-    QMap<QString, QString> documentProperties();
+    void loadPreview(const QString &chunks, const QString &dirty, bool enable, Mlt::Playlist &playlist);
 
     /** @brief Change track compsiting mode */
-    void switchCompositing(int mode);
+    void switchCompositing(bool enable);
 
     /** @brief Change a clip item's speed in timeline */
     Q_INVOKABLE void changeItemSpeed(int clipId, double speed);
+    /** @brief Activate time remap on the clip */
+    void remapItemTime(int clipId);
     /** @brief Delete selected zone and fill gap by moving following clips
      *  @param lift if true, the zone will simply be deleted but clips won't be moved
      */
@@ -553,7 +634,7 @@ public:
      *  @returns the zone end position or -1 on fail
      */
     Q_INVOKABLE bool insertClipZone(const QString &binId, int tid, int pos);
-    int insertZone(const QString &binId, QPoint zone, bool overwrite);
+    int insertZone(const QString &binId, QPoint zone, bool overwrite, Fun &undo, Fun &redo);
     void updateClip(int clipId, const QVector<int> &roles);
     void showClipKeyframes(int clipId, bool value);
     void showCompositionKeyframes(int clipId, bool value);
@@ -562,24 +643,27 @@ public:
     /** @brief timeline preview params changed, reset */
     void resetPreview();
     /** @brief Set target tracks (video, audio) */
-    void setTargetTracks(bool hasVideo, QMap <int, QString> audioTargets);
+    void setTargetTracks(bool hasVideo, const QMap <int, QString> &audioTargets);
     /** @brief Restore Bin Clip original target tracks (video, audio) */
     void restoreTargetTracks();
     /** @brief Return asset's display name from it's id (effect or composition) */
     Q_INVOKABLE const QString getAssetName(const QString &assetId, bool isTransition);
     /** @brief Set keyboard grabbing on current selection */
     Q_INVOKABLE void grabCurrent();
+    /** @brief Returns true if an item is currently grabbed (has keyboard focus) */
+    bool grabIsActive() const;
     /** @brief Returns keys for all used thumbnails */
-    QStringList getThumbKeys();
+    const std::unordered_map<QString, std::vector<int>> getThumbKeys();
     /** @brief Returns true if a drag operation is currently running in timeline */
     bool dragOperationRunning();
+    /** @brief Returns true if the timeline is in trimming mode (slip, slide, ripple, rolle) */
+    bool trimmingActive();
     /** @brief Disconnect some stuff before closing project */
     void prepareClose();
     /** @brief Check that we don't keep a deleted track id */
     void checkTrackDeletion(int selectedTrackIx);
     /** @brief Return true if an overlay track is used */
     bool hasPreviewTrack() const;
-    void updatePreviewConnection(bool enable);
     /** @brief Display project master effects */
     Q_INVOKABLE void showMasterEffects();
     /** @brief Return true if an instance of this bin clip is currently under timeline cursor */
@@ -600,23 +684,13 @@ public:
     /** @brief Add tracks to project */
     void addTracks(int videoTracks, int audioTracks);
     /** @brief Get in/out of currently selected items */
-    QPoint selectionInOut() const;
+    std::pair<int, int> selectionInOut() const;
     /** @brief Create a mix transition with currently selected clip. If delta = -1, mix with previous clip, +1 with next clip and 0 will check cursor position*/
     Q_INVOKABLE void mixClip(int cid = -1, int delta = 0);
     /** @brief Temporarily un/plug a list of clips in timeline. */
-    void temporaryUnplug(QList<int> clipIds, bool hide);
-    /** @brief Edit the subtitle text*/
-    Q_INVOKABLE void editSubtitle(int startFrame, int endFrame, QString newText, QString oldText);
-    /** @brief Edit the subtitle end */
-    Q_INVOKABLE void resizeSubtitle(int startFrame, int endFrame, int oldEndFrame, bool refreshModel);
-    /** @brief Add subtitle clip at cursor's position in timeline */
-    Q_INVOKABLE void addSubtitle(int startframe = -1, QString text = QString());
-    /** @brief Cut a subtitle and split the text at @param pos */
-    void cutSubtitle(int id, int cursorPos);
-    /** @brief Delete subtitle clip with frame as start position*/
-    Q_INVOKABLE void deleteSubtitle(int frameframe, int endframe, QString Ctext);
+    void temporaryUnplug(const QList<int> &clipIds, bool hide);
     /** @brief Import a subtitle file*/
-    void importSubtitle(const QString path = QString());
+    void importSubtitle(const QString &path = QString());
     /** @brief Export a subtitle file*/
     void exportSubtitle();
     /** @brief Launch speech recognition on timeline zone*/
@@ -624,13 +698,28 @@ public:
     /** @brief Show active effect zone for current effect*/
     void showRulerEffectZone(QPair <int, int>inOut, bool checked);
     /** @brief Set the list of master effect zones */
-    void updateMasterZones(QVariantList zones);
+    void updateMasterZones(const QVariantList &zones);
     /** @brief get Maximum duration of a clip */
     int clipMaxDuration(int cid);
+    /** @brief Get Mix cut pos (the duration of the mix on the right clip) */
+    int getMixCutPos(int cid) const;
+    /** @brief Get align info for a mix. */
+    MixAlignment getMixAlign(int cid) const;
+    /** @brief Process a lift operation for multitrack operation. */
+    void processMultitrackOperation(int tid, int in);
+    /** @brief Save all sequence properties (timeline position, guides, groups, ..) to the timeline tractor. */
+    void getSequenceProperties(QMap<QString, QString> &seqProps);
+    /** @brief Trigger refresh of subtitles combo menu. */
+    void refreshSubtitlesComboIndex();
+    /** @brief Return the active subtitle layer */
+    int activeSubLayer() const;
+    /** @brief Set the active subtitle layer */
+    void setActiveSubLayer(int layer);
 
-public slots:
+public Q_SLOTS:
+    void updateClipActions();
     void resetView();
-    void setAudioTarget(QMap<int, int> tracks);
+    void setAudioTarget(const QMap<int, int> &tracks);
     Q_INVOKABLE void switchAudioTarget(int trackId);
     Q_INVOKABLE void setVideoTarget(int track);
     Q_INVOKABLE void setActiveTrack(int track);
@@ -639,53 +728,79 @@ public slots:
     void disablePreview(bool disable);
     void invalidateItem(int cid);
     void invalidateTrack(int tid);
-    void invalidateZone(int in, int out);
     void checkDuration();
     /** @brief Dis / enable multi track view. */
     void slotMultitrackView(bool enable = true, bool refresh = true);
     /** @brief Activate a video track by its position (0 = topmost). */
-    void activateTrackAndSelect(int trackPosition);
+    void activateTrackAndSelect(int trackPosition, bool notesMode = false);
     /** @brief Save timeline selected clips to target folder. */
     void saveTimelineSelection(const QDir &targetDir);
     /** @brief Restore timeline scroll pos on open. */
     void setScrollPos(int pos);
+    /** @brief Request resizing currently selected mix. */
+    void resizeMix(int cid, int duration, MixAlignment align, int leftFrames = -1);
     /** @brief change zone info with undo. */
     Q_INVOKABLE void updateZone(const QPoint oldZone, const QPoint newZone, bool withUndo = true);
     Q_INVOKABLE void updateEffectZone(const QPoint oldZone, const QPoint newZone, bool withUndo = true);
+    void updateTrimmingMode();
+    /** @brief When a clip or composition is moved, inform asset panel to update cursor position in keyframe views. */
+    void checkClipPosition(const QModelIndex &topLeft, const QModelIndex &, const QVector<int> &roles);
+    /** @brief Adjust all tracks height to fit in view. */
+    Q_INVOKABLE void autofitTrackHeight(int timelineHeight, int collapsedHeight);
+    Q_INVOKABLE void subtitlesMenuActivatedAsync(int ix);
+    /** @brief Switch the active subtitle in the list. */
+    void subtitlesMenuActivated(int ix);
+    /** @brief Select a clip in the next track that has the same effect as current clip. */
+    void switchFocusClip();
+    /** @brief Enable build in transform in selected clip. */
+    void enableBuildInTransform();
+    /** @brief Open the Subtitle Manager */
+    void showSubtitleManager(int page = 0);
+    Q_INVOKABLE void setTimelineMouseOffset(int offset);
 
-private slots:
-    void updateClipActions();
+private Q_SLOTS:
     void updateVideoTarget();
     void updateAudioTarget();
     /** @brief Dis / enable multi track view. */
     void updateMultiTrack();
+    /** @brief An operation was attempted on a locked track, animate lock icon to make user aware */
+    void slotFlashLock(int trackId);
+    void initializePreview();
+    /** @brief Display the active subtitle mode in subtitle track combobox. */
+    void loadSubtitleIndex();
 
 public:
     /** @brief a list of actions that have to be enabled/disabled depending on the timeline selection */
     QList<QAction *> clipActions;
+    /** @brief The in point for a multicam operation */
+    int multicamIn;
+    /** @brief Set the in point for a multicam operation and trigger necessary signals */
+    void setMulticamIn(int pos);
+    int timelineMouseOffset() { return m_timelineMouseOffset; };
 
 private:
+    int m_duration;
     QQuickItem *m_root;
     KActionCollection *m_actionCollection;
     std::shared_ptr<TimelineItemModel> m_model;
     bool m_usePreview;
     int m_audioTarget;
     int m_videoTarget;
-    int m_activeTrack;
     int m_audioRef;
+    int m_timelineMouseOffset{0};
     int m_hasAudioTarget {0};
     bool m_hasVideoTarget {false};
     int m_lastVideoTarget {-1};
+    TargetTracksData m_targetTrackInfo;
     /** @brief The last combination of audio targets in the form: {timeline track id, bin stream index} */
     QMap <int, int> m_lastAudioTarget;
     bool m_videoTargetActive {true};
     bool m_audioTargetActive {true};
-    QPair<int, int> m_recordStart;
-    int m_recordTrack;
+    std::pair<int, int> m_recordStart;
+    int m_recordTrack{-1};
     QPoint m_zone;
+    int m_activeTrack;
     double m_scale;
-    static int m_duration;
-    PreviewManager *m_timelinePreview;
     QAction *m_disablePreview;
     std::shared_ptr<AudioCorrelation> m_audioCorrelator;
     QMutex m_metaMutex;
@@ -695,14 +810,18 @@ private:
     QMetaObject::Connection m_connection;
     QMetaObject::Connection m_deleteConnection;
     QPoint m_effectZone;
+    bool m_autotrackHeight;
     QVariantList m_masterEffectZones;
+    /** @brief The clip that is displayed in the preview monitor during a trimming operation*/
+    int m_trimmingMainClip;
+    /** @brief The position of the active subtitle in the menu list*/
+    int m_activeSubPosition{-1};
 
-    void initializePreview();
-    bool darkBackground() const;
     int getMenuOrTimelinePos() const;
+    /** @brief Prepare the preview manager */
+    void connectPreviewManager();
 
-signals:
-    void selected(Mlt::Producer *producer);
+Q_SIGNALS:
     void selectionChanged();
     void selectedMixChanged();
     void frameFormatChanged();
@@ -710,7 +829,7 @@ signals:
     void scaleFactorChanged();
     void audioThumbFormatChanged();
     void audioThumbNormalizeChanged();
-    void durationChanged();
+    void durationChanged(int duration);
     void audioTargetChanged();
     void videoTargetChanged();
     void hasAudioTargetChanged();
@@ -718,20 +837,28 @@ signals:
     void lastAudioTargetChanged();
     void autoScrollChanged();
     void lastVideoTargetChanged();
+    void timelineMouseOffsetChanged(int);
     void activeTrackChanged();
+    void activeSubLayerChanged();
+    void trimmingMainClipChanged();
     void colorsChanged();
     void showThumbnailsChanged();
     void showAudioThumbnailsChanged();
     void showMarkersChanged();
     void rippleChanged();
     void scrubChanged();
+    void subtitlesWarningChanged();
+    void multicamInChanged();
+    void autotrackHeightChanged();
     void seeked(int position);
     void zoneChanged();
+    void subtitlesListChanged();
+    void maxSubLayerChanged();
     void zoneMoved(const QPoint &zone);
     /** @brief Requests that a given parameter model is displayed in the asset panel */
     void showTransitionModel(int tid, std::shared_ptr<AssetParameterModel>);
     /** @brief Requests that a given mix is displayed in the asset panel */
-    void showMixModel(int cid, const std::shared_ptr<AssetParameterModel> &asset);
+    void showMixModel(int cid, const std::shared_ptr<AssetParameterModel> &asset, bool refreshOnly);
     void showItemEffectStack(const QString &clipName, std::shared_ptr<EffectStackModel>, QSize frameSize, bool showKeyframes);
     void showSubtitle(int id);
     /** @brief notify of chunks change
@@ -753,10 +880,12 @@ signals:
     /** @brief Center timeline view on current position
      */
     void centerView();
+    void guidesLockedChanged();
     void effectZoneChanged();
     void masterZonesChanged();
     Q_INVOKABLE void ungrabHack();
     void regainFocus();
+    void updateAssetPosition(int itemId, const QUuid uuid);
+    void stopAudioRecord();
+    void activeSubtitlePositionChanged();
 };
-
-#endif
